﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using DxLibDLL;

namespace Ryujin
{
    /// <summary>
    /// 背景を描画するクラスです。
    /// </summary>
    class GraphBack
    {
        /// <summary>
        /// プライベートなコンストラクタ。
        /// </summary>
        private GraphBack()
        {
            ; // 何もしない
        }

        /// <summary>
        /// ドン揺れと描画エリアを勘案して、画像を描画します。
        /// </summary>
        /// <param name="xd">X座標</param>
        /// <param name="yd">Y座標</param>
        /// <param name="GrHandle">描画するグラフィックのハンドル</param>
        /// <param name="TransFlag">1のとき透明色を有効にする</param>
        /// <returns>成功の場合 0、エラー発生の場合 -1</returns>
        private static int DrawGraphFd(int xd, int yd, int GrHandle, int TransFlag)
        {
            return DX.DrawGraph(xd + Setting.FX + GV.dn.x, yd + Setting.FY + GV.dn.y, GrHandle, TransFlag);
        }

        /// <summary>
        /// 背景番号 0。通常の背景を描画します。
        /// </summary>
        private static void graph_back_00()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            DX.SetDrawArea(Setting.FX, Setting.FY, Setting.FX + Setting.FMX, Setting.FY + Setting.FMY); // 描画可能エリアを設定
            DrawGraphFd(0, GV.count % 700 - 700, imageManager.img_back[0], 0);
            DrawGraphFd(0, GV.count % 700, imageManager.img_back[0], 0);
            DX.SetDrawArea(0, 0, 640, 480); // エリアを戻す
        }

        /// <summary>
        /// 背景番号 1。スペルカード用の背景を描画します。
        /// </summary>
        private static void graph_back_01()
        {
            ImageManager imageManager = ImageManager.GetInstance();
            DX.SetDrawArea(Setting.FX, Setting.FY, Setting.FX + Setting.FMX, Setting.FY + Setting.FMY); // 描画可能エリアを設定

            DrawGraphFd(0, -(GV.count % 480), imageManager.img_back[11], 0);
            DrawGraphFd(0, -(GV.count % 480) + 480, imageManager.img_back[11], 0);
            DrawGraphFd(0, 0, imageManager.img_back[10], 1);

            DX.SetDrawArea(0, 0, 640, 480); // エリアを戻す
        }

        /// <summary>
        /// 背景を描画するメイン処理です。
        /// </summary>
        public static void graph_back_main()
        {
            // ボスがオンでスペルカードなら
            if (GV.boss.flag == 1 && GV.boss.back_knd[GV.boss.knd] == 1)
                graph_back_01();
            else
                graph_back_00();
        }

    }
}
