﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// 自キャラクタのショットを表すクラスです。
    /// </summary>
    class CShot
    {
        /// <summary>
        /// 敵の当たり判定範囲。
        /// TODO 暫定でここに置きます。
        /// </summary>
        private int[] enemy_range = new int[] { 16, 30, 16, 50 };

        /// <summary>
        /// 自キャラクタのショットの当たり判定範囲。
        /// </summary>
        private int[] cshot_range = new int[] { 6, 0 };

        /// <summary>
        /// フラグ。
        /// </summary>
        public int flag;

        /// <summary>
        /// パワー。
        /// </summary>
        public int power;

        /// <summary>
        /// カウンタ。
        /// </summary>
        public int cnt;

        /// <summary>
        /// 種類。
        /// </summary>
        public int knd;

        /// <summary>
        /// 画像。
        /// </summary>
        public int img;

        /// <summary>
        /// X座標。
        /// </summary>
        public double x;
        
        /// <summary>
        /// Y座標。
        /// </summary>
        public double y;

        /// <summary>
        /// 角度。
        /// </summary>
        public double angle;

        /// <summary>
        /// スピード。
        /// </summary>
        public double spd;
        
        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            this.flag = 0;
            this.power = 0;
            this.cnt = 0;
            this.knd = 0;
            this.img = 0;
            this.x = 0.0;
            this.y = 0.0;
            this.angle = 0.0;
            this.spd = 0.0;
        }
        
        /// <summary>
        /// 敵に当たったかどうか判定する。
        /// </summary>
        /// <param name="enemy">敵</param>
        /// <returns>当たった場合は true</returns>
        private bool out_judge_cshot(Enemy enemy)
        {
            if (this.cnt > 0) // ショットの軌道が一度でも計算されていたら
            {
                return MyMath.out_judge(this.x, this.y, enemy.x, enemy.y, cshot_range[this.knd], enemy_range[enemy.knd], this.spd, this.angle);
            }
            return false;
        }

        /// <summary>
        /// ボスに当たったかどうか判定する。
        /// </summary>
        /// <param name="enemy">ボス</param>
        /// <returns>当たった場合は true</returns>
        private bool out_judge_cshot_boss(Boss boss)
        {
            if (this.cnt > 0) // ショットの軌道が一度でも計算されていたら
            {
                return MyMath.out_judge(this.x, this.y, boss.x, boss.y, cshot_range[this.knd], Setting.BRANGE, this.spd, this.angle);
            }
            return false;
        }

        /// <summary>
        /// 自キャラクタのショットが敵に当たる処理をします。
        /// </summary>
        public void cshot_and_enemy()
        {
            for (int s = 0; s < GV.enemys.length(); s++) // 敵の総数
            {
                Enemy enemy = GV.enemys[s];
                if (enemy.flag > 0)
                {
                    if (this.out_judge_cshot(enemy)) // 自キャラクタのショットと敵が当たっていたら
                    {
                        this.flag = 0; // 自キャラクタのショットを消す
                        enemy.hit_enemy(this.power); // 敵に対して処理する
                        return; // 消したショットでボスを判定しないために原作では break; のところを return; にする
                    }
                }
            }

            // ボスが出現していて、描画しないフラグがオフで、ショット中なら
            if (GV.boss.flag == 1 && GV.boss.graph_flag == 0 && GV.boss.state == 2)
            {
                if(this.out_judge_cshot_boss(GV.boss))
                {
                    this.flag = 0; // 自キャラクタのショットを消す

                    MusicManager musicManager = MusicManager.GetInstance();
                    musicManager.se_flag[8] = 1; // ボスに当たった音(勝手に実装)

                    GV.boss.hit_boss(this.power); // ボスに対して処理する
                }
            }
        }

        /// <summary>
        /// 一番近い敵を探して角度をセットします。
        /// </summary>
        private void calc_homing()
        {
            int d; // ショットと敵の距離
            int min = -1; // 最小値
            double x;
            double y;
            Enemy target_enemy = null; // 対象の敵

            if (GV.boss.flag == 0) // ボスが居ない時
            {
                for (int i = 0; i < GV.enemys.length(); i++) // 敵の総数分
                {
                    Enemy enemy = GV.enemys[i];
                    if (enemy.flag == 1)
                    {
                        x = enemy.x - this.x;
                        y = enemy.y - this.y;
                        d = (int)(x * x + y * y); // ショットと敵の距離
                        if (d < min || min == -1) // 計算結果が最小値かまだ格納していないなら
                        {
                            target_enemy = enemy; // 対象の敵を記録
                            min = d; // 距離記録
                        }
                    }
                }
            }

            // 近い敵が見つかったら、あるいはボスがいて、HPがあるときは角度をセット
            if (target_enemy != null || (GV.boss.flag == 1 && GV.boss.hp > 0))
            {
                if (GV.boss.flag == 0)
                {
                    x = target_enemy.x - this.x;
                    y = target_enemy.y - this.y;
                }
                else
                {
                    x = GV.boss.x - this.x;
                    y = GV.boss.y - this.y;
                }

                this.angle = Math.Atan2(y, x);
            }
        }

        /// <summary>
        /// ショットの移動計算をします。
        /// </summary>
        public void calc_cshot()
        {
            // 画面外判定用の変数(11, 55はショットの画像サイズ)
            int dranx = (int)(this.spd + 11 / 2);
            int drany = (int)(this.spd + 55 / 2);

            if (this.knd == 1) // ホーミング弾ならば
                this.calc_homing();

            this.x += Math.Cos(this.angle) * this.spd;
            this.y += Math.Sin(this.angle) * this.spd;
            this.cnt++;

            if (this.x < -dranx || Setting.FMX + dranx < this.x || this.y < -drany || Setting.FMY + drany < this.y) // 画面から外れたら
                this.flag = 0;
        }
    }
}
