﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// 敵の行動パターンを定義するクラスです。
    /// </summary>
    class EnemyPattern
    {
        /// <summary>
        /// プライベートなコンストラクタ。
        /// </summary>
        private EnemyPattern()
        {
            ; // 何もしない
        }

        /// <summary>
        /// 移動パターン 0。
        /// 下がってきて停滞して上がっていく。
        /// </summary>
        /// <param name="enemy">敵</param>
        private static void pattern0(Enemy enemy)
        {
            if (enemy.cnt == 0)
                enemy.vy = 3.0; // 下がってくる
            if (enemy.cnt == 40)
                enemy.vy = 0.0; // 止まる
            if (enemy.cnt == 40 + enemy.wait) // 登録された時間だけ停滞して
                enemy.vy = -3.0; // 上がっていく
        }

        /// <summary>
        /// 移動パターン 1。
        /// 下がってきて停滞して左下に行く。
        /// </summary>
        /// <param name="enemy">敵</param>
        private static void pattern1(Enemy enemy)
        {
            if (enemy.cnt == 0)
                enemy.vy = 3.0; // 下がってくる
            if (enemy.cnt == 40)
                enemy.vy = 0.0; // 止まる
            if (enemy.cnt == 40 + enemy.wait) // 登録された時間だけ停滞して
            {
                enemy.vx = -1.0; // 左へ下がっていく
                enemy.vy = 2.0;
                enemy.muki = 0; // 左向きセット
            }
        }

        /// <summary>
        /// 移動パターン 2。
        /// 下がってきて停滞して右下に行く。
        /// </summary>
        /// <param name="enemy">敵</param>
        private static void pattern2(Enemy enemy)
        {
            if (enemy.cnt == 0)
                enemy.vy = 3.0; // 下がってくる
            if (enemy.cnt == 40)
                enemy.vy = 0.0; // 止まる
            if (enemy.cnt == 40 + enemy.wait) // 登録された時間だけ停滞して
            {
                enemy.vx = 1.0; // 右へ下がっていく
                enemy.vy = 2.0;
                enemy.muki = 0; // 右向きセット
            }
        }

        /// <summary>
        /// 移動パターン 3。
        /// すばやく降りてきて左へ。
        /// </summary>
        /// <param name="enemy">敵</param>
        private static void pattern3(Enemy enemy)
        {
            if (enemy.cnt == 0)
                enemy.vy = 5.0;
            if (enemy.cnt == 30) // 途中で左向きに
                enemy.muki = 0;
            if (enemy.cnt < 100)
            {
                enemy.vx -= 5.0 / 100.0; // 左向き加速
                enemy.vy -= 5.0 / 100.0; // 減速
            }
        }

        /// <summary>
        /// 移動パターン 4。
        /// すばやく降りてきて右へ。
        /// </summary>
        /// <param name="enemy">敵</param>
        private static void pattern4(Enemy enemy)
        {
            if (enemy.cnt == 0)
                enemy.vy = 5.0;
            if (enemy.cnt == 30) // 途中で右向きに
                enemy.muki = 2;
            if (enemy.cnt < 100)
            {
                enemy.vx += 5.0 / 100.0; // 右向き加速
                enemy.vy -= 5.0 / 100.0; // 減速
            }
        }

        /// <summary>
        /// 移動パターン 5。
        /// 斜め左下へ。
        /// </summary>
        /// <param name="enemy">敵</param>
        private static void pattern5(Enemy enemy)
        {
            if (enemy.cnt == 0)
            {
                enemy.vx -= 1.0;
                enemy.vy = 2.0;
                enemy.muki = 0;
            }
        }

        /// <summary>
        /// 移動パターン 6。
        /// 斜め右下へ。
        /// </summary>
        /// <param name="enemy">敵</param>
        private static void pattern6(Enemy enemy)
        {
            if (enemy.cnt == 0)
            {
                enemy.vx += 1.0;
                enemy.vy = 2.0;
                enemy.muki = 2;
            }
        }

        /// <summary>
        /// 移動パターン 7。
        /// 停滞してそのまま左上に。
        /// </summary>
        /// <param name="enemy">敵</param>
        private static void pattern7(Enemy enemy)
        {
            if (enemy.cnt == enemy.wait) // 登録された時間だけ停滞して
            {
                enemy.vx = -0.7; // 左へ上がっていく
                enemy.vy = -0.3;
                enemy.muki = 0; // 左向き
            }
        }

        /// <summary>
        /// 移動パターン 8。
        /// 停滞してそのまま右上に。
        /// </summary>
        /// <param name="enemy">敵</param>
        private static void pattern8(Enemy enemy)
        {
            if (enemy.cnt == enemy.wait) // 登録された時間だけ停滞して
            {
                enemy.vx = 0.7; // 右へ上がっていく
                enemy.vy = -0.3;
                enemy.muki = 2; // 右向き
            }
        }

        /// <summary>
        /// 移動パターン 9。
        /// 停滞してそのまま上に。
        /// </summary>
        /// <param name="enemy">敵</param>
        private static void pattern9(Enemy enemy)
        {
            if (enemy.cnt == enemy.wait) // 登録された時間だけ停滞して
            {
                enemy.vy = -1.0;
            }
        }

        /// <summary>
        /// 下がってきてウロウロして上がっていく。
        /// </summary>
        /// <param name="enemy">敵</param>
        private static void pattern10(Enemy enemy)
        {
            if (enemy.cnt == 0)
                enemy.vy = 4.0; // 下がってくる
            if (enemy.cnt == 40)
                enemy.vy = 0.0; // 止まる
            if (enemy.cnt >= 40)
            {
                if (enemy.cnt % 60 == 0)
                {
                    int r = (Math.Cos(enemy.ang) < 0.0) ? 0 : 1;
                    enemy.sp = 6.0 + MyMath.rang(2.0);
                    enemy.ang = MyMath.rang(Math.PI / 4.0) + Math.PI * r;
                    enemy.muki = 2 - 2 * r;
                }
                enemy.sp *= 0.95;
            }
            if (enemy.cnt >= 40 + enemy.wait)
                enemy.vy -= 0.05;
        }

        /// <summary>
        /// パターンを実行します。
        /// </summary>
        /// <param name="enemy">敵</param>
        public static void do_pattern(Enemy enemy)
        {
            switch (enemy.pattern)
            {
                case 0:
                    pattern0(enemy);
                    break;
                case 1:
                    pattern1(enemy);
                    break;
                case 2:
                    pattern2(enemy);
                    break;
                case 3:
                    pattern3(enemy);
                    break;
                case 4:
                    pattern4(enemy);
                    break;
                case 5:
                    pattern5(enemy);
                    break;
                case 6:
                    pattern6(enemy);
                    break;
                case 7:
                    pattern7(enemy);
                    break;
                case 8:
                    pattern8(enemy);
                    break;
                case 9:
                    pattern9(enemy);
                    break;
                case 10:
                    pattern10(enemy);
                    break;
                default:
                    Debug.WriteLine("Enemyの patternの値が " + enemy.pattern + "です。");
                    break;
            }
        }

    }
}
