﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// 設定を格納するクラスです。
    /// </summary>
    class Setting
    {
        /// <summary>
        /// ベースディレクトリ。
        /// </summary>
        public const string BASE_DIR = "../../dat/";

        /// <summary>
        /// デバッグモード。
        /// 0: 何も表示しない
        /// 1: FPSを表示する
        /// 2: 上記に加えて、各種処理時間を表示する
        /// 3: 上記に加えて、当たり判定などを表示する
        /// </summary>
        public const int DEBUG_MODE = 2;

        /// <summary>
        /// フィールド X座標最大値。
        /// </summary>
        public const int FMX = 384;

        /// <summary>
        /// フィールド Y座標最大値。
        /// </summary>
        public const int FMY = 448;

        /// <summary>
        /// フィールド X座標最小値。
        /// </summary>
        public const int FX = 32;

        /// <summary>
        /// フィールド Y座標最小値。
        /// </summary>
        public const int FY = 16;
        
        /// <summary>
        /// パッドキーの最大数。
        /// </summary>
        public const int PAD_MAX = 16;

        /// <summary>
        /// 敵の同時表示最大数。
        /// </summary>
        public const int ENEMY_MAX = 30;

        /// <summary>
        /// 敵の出現情報の最大数。
        /// </summary>
        public const int ENEMY_ORDER_MAX = 500;

        /// <summary>
        /// 弾幕が持つ弾の最大数。
        /// </summary>
        public const int SHOT_BULLET_MAX = 1000;

        /// <summary>
        /// 一度に画面に表示できる敵の弾幕の最大数。
        /// </summary>
        public const int SHOT_MAX = 30;

        /// <summary>
        /// 自キャラクタのショットの登録最大数。
        /// </summary>
        public const int CSHOT_MAX = 200;

        /// <summary>
        /// 効果音の種類の最大数。
        /// </summary>
        public const int SE_MAX = 100;

        /// <summary>
        /// エフェクトの登録最大数。
        /// </summary>
        public const int EFFECT_MAX = 100;

        /// <summary>
        /// 消滅エフェクトの登録最大数。
        /// </summary>
        public const int DEL_EFFECT_MAX = 30;

        /// <summary>
        /// 自キャラクタの大きさ(半径)。
        /// </summary>
        public const double CRANGE = 2.0;

        /// <summary>
        /// ?
        /// </summary>
        public const double CRANGE11 = 5.0;

        /// <summary>
        /// ボスの定位置 X座標。
        /// </summary>
        public const double BOSS_POS_X = FMX / 2;

        /// <summary>
        /// ボスの定位置 Y座標。
        /// </summary>
        public const double BOSS_POS_Y = FMY / 4;

        /// <summary>
        /// ボスが持つ弾の最大数。
        /// </summary>
        public const int BOSS_BULLET_MAX = 3000;

        /// <summary>
        /// ボスの大きさ(半径)。
        /// </summary>
        public const double BRANGE = 40.0;

        /// <summary>
        /// 弾幕最大数。
        /// </summary>
        public const int DANMAKU_MAX = 50;

        /// <summary>
        /// レーザー最大数。
        /// </summary>
        public const int LAZER_MAX = 100;

        /// <summary>
        /// 子の最大数。
        /// </summary>
        public const int CHILD_MAX = 20;

        /// <summary>
        /// ステージ数。
        /// </summary>
        public const int STAGE_NUM = 5;

        /// <summary>
        /// アイテム表示最大数。
        /// </summary>
        public const int ITEM_MAX = 100;

        /// <summary>
        /// アイテムゲットボーダーライン。
        /// </summary>
        public const int ITEM_GET_BORDER_LINE = 100;

        /// <summary>
        /// アイテムの吸収範囲。
        /// </summary>
        public const int ITEM_INDRAW_RANGE = 70;

        /// <summary>
        /// アイテムの当たり判定。
        /// </summary>
        public const int ITEM_HIT_BOX = 20;

        /// <summary>
        /// ステージあたりに登場するボスの最大数。
        /// </summary>
        public const int BOSS_NUM = 3;

        /// <summary>
        /// ボスの持つ弾幕数の最大値。
        /// </summary>
        public const int BOSS_DANMAKU_MAX = 20;

        /// <summary>
        /// フェードインのカウント。
        /// </summary>
        public const int FADE_IN_COUNT = 30;

        /// <summary>
        /// フェードアウトのカウント。
        /// </summary>
        public const int FADE_OUT_COUNT = 90;

        /// <summary>
        /// ボスを倒してからステージクリアまでのカウント。
        /// </summary>
        public const int STAGE_CLEAR_COUNT = 200;

        /// <summary>
        /// 不明。原作では boss_shot.cppで宣言しているが、暫定的にここに記述する。
        /// </summary>
        public const double V0 = 10.0;

        /// <summary>
        /// プライベートなコンストラクタ。
        /// </summary>
        private Setting()
        {
            ; // 何もしない
        }
    }
}
