﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using DxLibDLL;

namespace Ryujin
{
    /// <summary>
    /// 画像群を管理するクラスです。
    /// </summary>
    class ImageManager
    {
        /// <summary>
        /// マネジャー。
        /// </summary>
        private static ImageManager manager = new ImageManager();

        /// <summary>
        /// ボード画像。
        /// </summary>
        public int[] img_board = new int[40];

        /// <summary>
        /// キャラクタ画像 9枚分。X2(変身用)。
        /// </summary>
        public int[,] img_ch = new int[2, 12];

        /// <summary>
        /// 自キャラクタショット用画像。
        /// </summary>
        public int[] img_cshot = new int[3];

        /// <summary>
        /// 敵画像 9枚分。X3体分。
        /// </summary>
        public int[,] img_enemy = new int[3, 9];

        /// <summary>
        /// 弾の画像。
        /// </summary>
        public int[,] img_bullet = new int[12, 10];

        /// <summary>
        /// レーザーの画像。
        /// </summary>
        public int[,] img_lazer = new int[10, 10];

        /// <summary>
        /// レーザーの元の画像。
        /// </summary>
        public int[,] img_lazer_moto = new int[10, 10];

        /// <summary>
        /// 消滅エフェクトの画像。
        /// </summary>
        public int[] img_del_effect = new int[5];

        /// <summary>
        /// 背景用画像。
        /// </summary>
        public int[] img_back = new int[20];

        /// <summary>
        /// ボムのエフェクト画像。
        /// </summary>
        public int[] img_eff_bom = new int[5];

        /// <summary>
        /// リリアのドット絵画像。
        /// </summary>
        public int[] img_dot_riria = new int[8];

        /// <summary>
        /// キャラクタに関するその他の画像。
        /// </summary>
        public int[] img_chetc = new int[10];

        /// <summary>
        /// その他の画像。
        /// </summary>
        public int[] img_etc = new int[50];

        /// <summary>
        /// アイテムの画像。
        /// </summary>
        public int[,] img_item = new int[6, 2];

        /// <summary>
        /// 数字の画像。
        /// </summary>
        public int[,] img_num = new int[3, 12];

        /// <summary>
        /// 表情の画像。
        /// </summary>
        public int[,] img_face = new int[1, 6];

        /// <summary>
        /// プライベートなコンストラクタ。
        /// </summary>
        private ImageManager()
        {
            ; // 何もしない
        }
        
        /// <summary>
        /// マネジャーを返します。
        /// </summary>
        /// <returns>マネジャー</returns>
        public static ImageManager GetInstance()
        {
            return manager;
        }

        /// <summary>
        /// 画像を読み込みます。
        /// </summary>
        public void load()
        {
            string baseDir = Setting.BASE_DIR;
            this.img_board[10] = DX.LoadGraph(baseDir + "img/board/10.png");
            this.img_board[11] = DX.LoadGraph(baseDir + "img/board/11.png");
            this.img_board[12] = DX.LoadGraph(baseDir + "img/board/12.png");
            this.img_board[20] = DX.LoadGraph(baseDir + "img/board/20.png");
            DX.LoadDivGraph(baseDir + "img/char/0.png", 12, 4, 3, 73, 73, out this.img_ch[0, 0]);
            this.img_cshot[0] = DX.LoadGraph(baseDir + "img/char/bl_00.png");
            this.img_cshot[1] = DX.LoadGraph(baseDir + "img/char/bl_01.png");
            this.img_cshot[2] = DX.LoadGraph(baseDir + "img/char/ball.png");
            DX.LoadDivGraph(baseDir + "img/enemy/0.png", 9, 3, 3, 32, 32, out this.img_enemy[0, 0]);
            DX.LoadDivGraph(baseDir + "img/bullet/b0.png", 5, 5, 1, 76, 76, out this.img_bullet[0, 0]);
            DX.LoadDivGraph(baseDir + "img/bullet/b1.png", 6, 6, 1, 22, 22, out this.img_bullet[1, 0]);
            DX.LoadDivGraph(baseDir + "img/bullet/b2.png", 10, 10, 1, 5, 120, out this.img_bullet[2, 0]);
            DX.LoadDivGraph(baseDir + "img/bullet/b3.png", 5, 5, 1, 19, 34, out this.img_bullet[3, 0]);
            DX.LoadDivGraph(baseDir + "img/bullet/b4.png", 10, 10, 1, 38, 38, out this.img_bullet[4, 0]);
            DX.LoadDivGraph(baseDir + "img/bullet/b5.png", 3, 3, 1, 14, 16, out this.img_bullet[5, 0]);
            DX.LoadDivGraph(baseDir + "img/bullet/b6.png", 3, 3, 1, 14, 18, out this.img_bullet[6, 0]);
            DX.LoadDivGraph(baseDir + "img/bullet/b7.png", 10, 10, 1, 16, 16, out this.img_bullet[7, 0]);
            DX.LoadDivGraph(baseDir + "img/bullet/b8.png", 10, 10, 1, 12, 18, out this.img_bullet[8, 0]);
            DX.LoadDivGraph(baseDir + "img/bullet/b9.png", 3, 3, 1, 13, 19, out this.img_bullet[9, 0]);
            DX.LoadDivGraph(baseDir + "img/bullet/b10.png", 8, 8, 1, 8, 8, out this.img_bullet[10, 0]);
            DX.LoadDivGraph(baseDir + "img/bullet/b11.png", 8, 8, 1, 35, 32, out this.img_bullet[11, 0]);

            DX.LoadDivGraph(baseDir + "img/bullet/l0.png", 2, 2, 1, 30, 460, out this.img_lazer[0, 0]);
            DX.LoadDivGraph(baseDir + "img/bullet/l0_moto.png", 2, 2, 1, 70, 70, out this.img_lazer_moto[0, 0]);

            DX.LoadDivGraph(baseDir + "img/enemy/hit_effect.png", 5, 5, 1, 140, 140, out this.img_del_effect[0]);

            this.img_back[0] = DX.LoadGraph(baseDir + "img/back/0/back0.png");
            this.img_back[10] = DX.LoadGraph(baseDir + "img/back/10/kotei0.png");
            this.img_back[11] = DX.LoadGraph(baseDir + "img/back/10/scroll0.png");

            this.img_eff_bom[0] = DX.LoadGraph(baseDir + "img/effect/bom0.png");
            this.img_eff_bom[1] = DX.LoadGraph(baseDir + "img/effect/bom1.png");
            this.img_eff_bom[2] = DX.LoadGraph(baseDir + "img/char/body.png");
            this.img_eff_bom[3] = DX.LoadGraph(baseDir + "img/bullet/bom_title0.png");

            DX.LoadDivGraph(baseDir + "img/char/riria.png", 8, 8, 1, 100, 100, out this.img_dot_riria[0]);

            this.img_chetc[0] = DX.LoadGraph(baseDir + "img/char/atari.png"); // 当たり判定
            // this.img_chetc[2] = DX.LoadGraph(baseDir + "img/char/ball.png"); // ボール

            this.img_etc[1] = DX.LoadGraph(baseDir + "img/enemy/hp.png");
            this.img_etc[2] = DX.LoadGraph(baseDir + "img/enemy/bossback.png");
            this.img_etc[5] = DX.LoadGraph(baseDir + "img/enemy/bossback3.png");
            this.img_etc[6] = DX.LoadGraph(baseDir + "img/enemy/bossback4.png");
            this.img_etc[7] = DX.LoadGraph(baseDir + "img/enemy/hp_boss.png"); // ボスの HP
            this.img_etc[8] = DX.LoadGraph(baseDir + "img/board/hoshi.png"); // ☆

            DX.LoadDivGraph(baseDir + "img/item/p0.png", 2, 2, 1, 35, 35, out this.img_item[0, 0]);
            DX.LoadDivGraph(baseDir + "img/item/p1.png", 2, 2, 1, 35, 35, out this.img_item[1, 0]);
            DX.LoadDivGraph(baseDir + "img/item/p2.png", 2, 2, 1, 15, 15, out this.img_item[2, 0]);
            DX.LoadDivGraph(baseDir + "img/item/p3.png", 2, 2, 1, 35, 35, out this.img_item[3, 0]);
            DX.LoadDivGraph(baseDir + "img/item/p4.png", 2, 2, 1, 35, 35, out this.img_item[4, 0]);
            DX.LoadDivGraph(baseDir + "img/item/p5.png", 2, 2, 1, 35, 35, out this.img_item[5, 0]);

            DX.LoadDivGraph(baseDir + "img/num/2.png", 10, 10, 1, 16, 18, out this.img_num[0, 0]); // 数字

            this.img_face[0, 0] = DX.LoadGraph(baseDir + "img/char/face0.png");
            this.img_face[0, 1] = DX.LoadGraph(baseDir + "img/char/face1.png");
            this.img_face[0, 2] = DX.LoadGraph(baseDir + "img/char/face2.png");
            this.img_face[0, 3] = DX.LoadGraph(baseDir + "img/char/face3.png");
            this.img_face[0, 4] = DX.LoadGraph(baseDir + "img/char/face4.png");
            this.img_face[0, 5] = DX.LoadGraph(baseDir + "img/char/face5.png");
        }
    }
}
