﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Diagnostics;

namespace Ryujin
{
    /// <summary>
    /// メッセージ群を管理するためのクラスです。
    /// </summary>
    class MessageGroup
    {
        /// <summary>
        /// メッセージ群。
        /// </summary>
        private Message[] messages = new Message[Setting.MESSAGE_MAX];

        /// <summary>
        /// 登録されたメッセージの最大数。
        /// </summary>
        public int max_num;

        /// <summary>
        /// 現在(または次に)表示するメッセージ番号。
        /// </summary>
        public int num;

        /// <summary>
        /// メッセージを画面に表示しているかどうか。
        /// </summary>
        public bool is_displayed;

        /// <summary>
        /// パッドのひとつ前の情報。
        /// </summary>
        public int pad_old;

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        public MessageGroup()
        {
            for (int i = 0; i < this.messages.Length; i++)
                this.messages[i] = new Message();

            // あとで CSV読み込みします。
            this.messages[0].appear_count = 60;
            this.messages[0].num = 0;
            this.messages[0].max_num = 3;
            this.messages[0].message_detail[0].right = 0;
            this.messages[0].message_detail[0].message1 = "あ";
            this.messages[0].message_detail[0].message2 = "い";
            this.messages[0].message_detail[1].right = 1;
            this.messages[0].message_detail[1].face = 1;
            this.messages[0].message_detail[1].message1 = "う";
            this.messages[0].message_detail[1].message2 = "え";
            this.messages[0].message_detail[2].face = 2;
            this.messages[0].message_detail[2].message1 = "お";
            this.messages[0].message_detail[2].message2 = "か";
            
            this.messages[1].appear_count = 1650;
            this.messages[1].num = 0;
            this.messages[1].max_num = 3;
            this.messages[1].message_detail[0].right = 0;
            this.messages[1].message_detail[0].message1 = "あああ";
            this.messages[1].message_detail[0].message2 = "いいい";
            this.messages[1].message_detail[1].right = 1;
            this.messages[1].message_detail[1].message1 = "ううう";
            this.messages[1].message_detail[1].message2 = "えええ";
            this.messages[1].message_detail[2].message1 = "おおお";
            this.messages[1].message_detail[2].message2 = "かかか";

            this.messages[2].appear_count = 1660;
            this.messages[2].num = 0;
            this.messages[2].max_num = 3;
            this.messages[2].message_detail[0].right = 0;
            this.messages[2].message_detail[0].message1 = "ああ";
            this.messages[2].message_detail[0].message2 = "いい";
            this.messages[2].message_detail[1].right = 1;
            this.messages[2].message_detail[1].message1 = "うう";
            this.messages[2].message_detail[1].message2 = "ええ";
            this.messages[2].message_detail[2].message1 = "おお";
            this.messages[2].message_detail[2].message2 = "かか";

            this.num = 0;
            this.max_num = 3;
        }

        /// <summary>
        /// 現在のメッセージ詳細を返します。
        /// </summary>
        /// <returns></returns>
        public MessageDetail getCurrentMessageDetail()
        {
            return this.messages[this.num].message_detail[this.messages[this.num].num];
        }

        /// <summary>
        /// メッセージのメイン処理です。
        /// </summary>
        public void message_main()
        {
            KeyManager keyManager = KeyManager.GetInstance();
            if (this.is_displayed && this.pad_old == 0 && keyManager.CheckStatePad(GV.configpad.shot) > 0)
            {
                Message message = this.messages[this.num];
                message.num++;
                if (message.num == message.max_num)
                {
                    this.num++;
                    this.is_displayed = false;
                }
            }

            this.pad_old = keyManager.CheckStatePad(GV.configpad.shot);

            if (this.num < this.max_num) // メッセージを出し切っていなくて
            {
                Message message = this.messages[this.num];
                if (message.appear_count == GV.stage_count) // カウントが一緒で
                {
                    if (message.num < message.max_num) // 読みかけのメッセージがあれば
                    {
                        if (GV.stage_count != GV.boss_infos[GV.boss.num].appear_count) // ボスでなければ
                        {
                            if (!this.is_displayed)
                            {
                                // 敵と弾を消す
                                GV.enemys.initialize();
                                GV.shots.initialize();
                            }
                            this.is_displayed = true;
                        }
                        else // ボスであれば
                        {
                            if (GV.boss.cnt >= 60)
                                this.is_displayed = true;
                        }
                    }
                }
            }
        }
    }
}
