﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// 敵を表すクラスです。
    /// </summary>
    class Enemy
    {
        /// <summary>
        /// フラグ。
        /// </summary>
        public int flag;

        /// <summary>
        /// カウンタ。
        /// </summary>
        public int cnt;

        /// <summary>
        /// 移動パターン。
        /// </summary>
        public int pattern;

        /// <summary>
        /// 向き。
        /// </summary>
        public int muki;

        /// <summary>
        /// 敵の種類。
        /// </summary>
        public int knd;

        /// <summary>
        /// HP。
        /// </summary>
        public int hp;

        /// <summary>
        /// HP最大値。
        /// </summary>
        public int hp_max;

        /// <summary>
        /// 落とすアイテム。
        /// </summary>
        public int[] item_n = new int[6];

        /// <summary>
        /// 画像。
        /// </summary>
        public int img;
        
        /// <summary>
        /// 背景色。
        /// </summary>
        public int back_col;

        /// <summary>
        /// X座標。
        /// </summary>
        public double x;

        /// <summary>
        /// Y座標。
        /// </summary>
        public double y;

        /// <summary>
        /// 速度 x成分。
        /// </summary>
        public double vx;

        /// <summary>
        /// 速度 y成分。
        /// </summary>
        public double vy;

        /// <summary>
        /// スピード。
        /// </summary>
        public double sp;

        /// <summary>
        /// 角度。
        /// </summary>
        public double ang;

        /// <summary>
        /// 弾幕開始時間。
        /// </summary>
        public int bltime;

        /// <summary>
        /// 弾幕の種類。
        /// </summary>
        public int blknd;

        /// <summary>
        /// 弾の種類。
        /// </summary>
        public int blknd2;

        /// <summary>
        /// 色。
        /// </summary>
        public int col;

        /// <summary>
        /// 状態。
        /// </summary>
        public int state;

        /// <summary>
        /// 待機時間。
        /// </summary>
        public int wtime;

        /// <summary>
        /// 停滞時間。
        /// </summary>
        public int wait;

        /// <summary>
        /// 初期化します。
        /// </summary>
        public void initialize()
        {
            this.flag = 0;
            this.cnt = 0;
            this.cnt = 0;
            this.pattern = 0;
            this.muki = 0;
            this.knd = 0;
            this.hp = 0;
            this.hp_max = 0;
            for (int i = 0; i < item_n.Length; i++)
                this.item_n[i] = 0;
            this.img = 0;
            this.back_col = 0;
            this.x = 0.0;
            this.y = 0.0;
            this.vx = 0.0;
            this.vy = 0.0;
            this.sp = 0.0;
            this.ang = 0.0;
            this.bltime = 0;
            this.blknd2 = 0;
            this.col = 0;
            this.state = 0;
            this.wtime = 0;
            this.wait = 0;
        }

        /// <summary>
        /// パターンを実行します。
        /// </summary>
        public void do_pattern()
        {
            EnemyPattern.do_pattern(this);
        }

        /// <summary>
        /// 弾幕を登録します。
        /// </summary>
        public void enter_shot()
        {
            Shot shot = GV.shots.search_unused_shot(); // フラグの立ってない弾幕を探す
            if (shot != null) // 未使用の弾幕があれば
            {
                shot.initialize(); // 初期化して登録する
                shot.flag = 1; // フラグを立てる
                shot.knd = this.blknd; // 弾の種類
                shot.parent_enemy = this; // どの敵から発射されたものか。
                shot.cnt = 0;
            }
        }

        /// <summary>
        /// 敵のアイテムを出現させます。
        /// </summary>
        private void enter_enemy_items()
        {
            GV.items.enter_item(this.x, this.y, this.item_n);
        }

        /// <summary>
        /// 敵が死ぬかどうかの判定をします。
        /// </summary>
        private void enemy_death_judge()
        {
            MusicManager musicManager = MusicManager.GetInstance();
            musicManager.se_flag[8] = 1; // 敵に当たった音

            if (this.hp < 0) // 敵の HPが 0未満になったら
            {
                this.flag = 0; // 敵を消滅させる
                musicManager.se_flag[1] = 1; // 敵のピチュリ音
                GV.del_effects.enter_del_effect(this); // 消滅エフェクトの登録
                this.enter_enemy_items(); // 敵のアイテムを出現させる
                GV.shots.death(this); // 敵弾幕の消滅処理
            }
        }

        /// <summary>
        /// 敵に powerを食らわします。
        /// </summary>
        /// <param name="power">パワー</param>
        public void hit_enemy(int power)
        {
            this.hp -= power;
            this.enemy_death_judge();
        }
    }
}
