﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// エフェクトのパターンを定義するクラスです。
    /// </summary>
    class EffectPattern
    {
        /// <summary>
        /// プライベートなコンストラクタ。
        /// </summary>
        private EffectPattern()
        {
            ; // 何もしない
        }

        /// <summary>
        /// パターン 0。敵の消滅エフェクト。
        /// </summary>
        /// <param name="effect"></param>
        private static void pattern0(Effect effect)
        {
            effect.cnt++;
            effect.r += 0.08; // エフェクトの大きさを段々大きくする
            if (effect.cnt > 10) // 10カウント以上になったら
                effect.brt -= 25; // 明るさを少なくする
            if (effect.cnt > 20) // 20カウント以上になったら
                effect.flag = 0; // 消す
        }

        /// <summary>
        /// パターン 1。ボムのエフェクト。
        /// </summary>
        /// <param name="effect"></param>
        private static void pattern1(Effect effect)
        {
            // 座標計算
            effect.x += Math.Cos(effect.mvang) * effect.spd;
            effect.y += Math.Sin(effect.mvang) * effect.spd;

            // スピード計算
            if (effect.cnt < 60)
                effect.spd -= 0.2 + effect.cnt * effect.cnt / 3000.0;
            if (effect.cnt == 60)
            {
                effect.spd = 0.0;
                MusicManager musicManager = MusicManager.GetInstance();
                musicManager.se_flag[15] = 1;
                GV.dn.enter_dn(11, 20); // ドン揺れを設定する。
            }

            // 明るさと大きさを計算
            effect.r += 0.015;
            if (effect.cnt < 51)
                effect.brt += 5.0;
            if (effect.cnt >= 60)
            {
                effect.r += 0.04;
                effect.brt -= 255 / 30.0;
            }

            // カウントアップと消去計算
            effect.cnt++;
            if (effect.cnt >= 90)
                effect.flag = 0;
        }

        /// <summary>
        /// パターン 2。ボムのエフェクト(キャラ)。
        /// </summary>
        /// <param name="effect"></param>
        private static void pattern2(Effect effect)
        {
            // 座標計算
            effect.x += Math.Cos(effect.mvang) * effect.spd;
            effect.y += Math.Sin(effect.mvang) * effect.spd;

            // 明るさ計算
            if (effect.cnt < 51)
                effect.brt += 4.0;
            if (effect.cnt > 130 - 51)
                effect.brt -= 4.0;

            // カウントアップと消去計算
            effect.cnt++;
            if (effect.cnt >= 130)
                effect.flag = 0;
        }

        /// <summary>
        /// パターン 3。ボムのエフェクト(ライン)。
        /// </summary>
        /// <param name="effect"></param>
        private static void pattern3(Effect effect)
        {
            // 座標計算
            effect.x += Math.Cos(effect.mvang) * effect.spd;
            effect.y += Math.Sin(effect.mvang) * effect.spd;

            // 明るさ計算
            if (effect.cnt < 51)
                effect.brt += 2.0;
            if (effect.cnt > 130 - 51)
                effect.brt -= 2.0;

            // カウントアップと消去計算
            effect.cnt++;
            if (effect.cnt >= 130)
                effect.flag = 0;
        }

        /// <summary>
        /// パターン 4。喰らいボムのエフェクト。
        /// </summary>
        /// <param name="effect"></param>
        private static void pattern4(Effect effect)
        {
            // 明るさ計算
            if (effect.cnt >= 6)
                effect.brt -= 255 / 6.0;
            effect.r += 0.12;
            effect.cnt++;
            if (effect.cnt >= 12 || GV.ch.flag != 1)
                effect.flag = 0;
        }

        /// <summary>
        /// パターンを実行します。
        /// </summary>
        /// <param name="effect">エフェクト</param>
        public static void do_pattern(Effect effect)
        {
            switch (effect.knd)
            {
                case 0:
                    pattern0(effect);
                    break;
                case 1:
                    pattern1(effect);
                    break;
                case 2:
                    pattern2(effect);
                    break;
                case 3:
                    pattern3(effect);
                    break;
                case 4:
                    pattern4(effect);
                    break;
                default:
                    Debug.WriteLine("effect設定エラー(" + effect.knd + ")");
                    break;
            }
        }
    }
}
