﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using DxLibDLL;

namespace Ryujin
{
    /// <summary>
    /// DXライブラリを用いてファイルを読み込むラッパークラスです。
    /// </summary>
    class DXFileReader
    {
        /// <summary>
        /// 1行の最大文字数。
        /// </summary>
        private int num;

        /// <summary>
        /// ファイルハンドル。
        /// </summary>
        private int handle;

        /// <summary>
        /// コンストラクタ。1行の最大文字数を 100に設定します。
        /// </summary>
        public DXFileReader()
        {
            this.num = 1000;
        }

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        /// <param name="num">1行の最大文字数</param>
        public DXFileReader(int num)
        {
            this.num = num;
        }

        /// <summary>
        /// ファイルを開きます。
        /// </summary>
        /// <param name="FileName">開くファイルのパス</param>
        /// <param name="ASync">非同期読み込みを行うか(1のとき非同期読み込み、0のとき同期読み込み)</param>
        /// <returns>trueは成功。falseはエラー発生</returns>
        public bool open(string FilePath, int ASync)
        {
            this.handle = DX.FileRead_open(FilePath, ASync);
            if (this.handle != 0)
                return true;
            else
                return false;
        }

        /// <summary>
        /// 1行読み込みます。
        /// </summary>
        /// <returns>文字列。nullはエラー発生。</returns>
        public string readLine()
        {
            StringBuilder builder = new StringBuilder(this.num);
            int ret = DX.FileRead_gets(builder, this.num, this.handle);
            if (ret == -1)
                return null;
            else
                return builder.ToString();
        }

        /// <summary>
        /// ファイルを閉じます。
        /// </summary>
        /// <returns>trueは成功。falseはエラー発生</returns>
        public bool close()
        {
            int ret = DX.FileRead_close(this.handle);
            if (ret == 0)
                return true;
            else
                return false;
        }
    }
}
