﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Diagnostics;

namespace Ryujin
{
    /// <summary>
    /// メッセージ群を管理するためのクラスです。
    /// </summary>
    class MessageGroup
    {
        /// <summary>
        /// メッセージ群。
        /// </summary>
        private Message[] messages = new Message[Setting.MESSAGE_MAX];

        /// <summary>
        /// 登録されたメッセージの最大数。
        /// </summary>
        public int max_num;

        /// <summary>
        /// 現在(または次に)表示するメッセージ番号。
        /// </summary>
        public int num;

        /// <summary>
        /// カウンタ。
        /// </summary>
        public int cnt;

        /// <summary>
        /// メッセージを画面に表示しているかどうか。
        /// </summary>
        public bool is_displayed;

        /// <summary>
        /// パッドのひとつ前の情報。
        /// </summary>
        public int pad_old;

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        public MessageGroup()
        {
            for (int i = 0; i < this.messages.Length; i++)
                this.messages[i] = new Message();
        }

        /// <summary>
        /// メッセージを読み込みます。
        /// </summary>
        public void load_message()
        {
            string fname = Setting.BASE_DIR + "csv/messageH0.csv";
            int n = 0; // メッセージ番号
            int appear_count = -1; // 出現カウンタ
            int lineNumber = 0; // ファイルの行数

            this.max_num = -1; // ボスの数
            this.num = 0;
            this.cnt = 0;
            this.pad_old = 0;
            this.is_displayed = false;

            DXFileReader reader = new DXFileReader();
            if(! reader.open(fname, 0))
            {
                Debug.WriteLine("ファイルオープンに失敗しました(" + fname + ")");
                return;
            }

            for (int i = 0; i < 2; i++) // 最初の 2行を読み飛ばす
            {
                reader.readLine();
                lineNumber++;
            }

            try {
                Message message;

                while (true)
                {
                    string line = reader.readLine(); // 1行読み込む
                    lineNumber++;

                    if (line == null) // ファイルの末端に到達していれば
                        break; // ループから抜ける

                    if (line[0] == '/') // スラッシュがあれば
                        continue; // 読み飛ばす

                    string[] items = line.Split(new Char[] { ',' }); // 文字列を分解する

                    if (appear_count != int.Parse(items[0]))
                    {
                        if (this.max_num != -1)
                        {
                            message = this.messages[this.max_num];
                            message.max_num = n;
                        }

                        appear_count = int.Parse(items[0]);
                        this.max_num++;
                        n = 0;

                        // 情報を格納する
                        message = this.messages[this.max_num];
                        message.appear_count = appear_count;
                        message.num = 0;
                    }
                    else
                        message = this.messages[this.max_num];

                    // 情報を格納する
                    message.message_detail[n].knd = int.Parse(items[1]);
                    message.message_detail[n].face = int.Parse(items[2]);
                    message.message_detail[n].right = int.Parse(items[3]);
                    message.message_detail[n].message1 = items[4];
                    message.message_detail[n].message2 = items[5];

                    n++;
                }

                // 読み終え時に最後のメッセージを設定する
                message = this.messages[this.max_num];
                message.max_num = n;

                // メッセージ数を設定する
                this.max_num++;
            }
            catch (FormatException)
            {
                Debug.WriteLine(lineNumber + "行目に不適切なデータが存在します");
            }

            // ファイルのクローズ
            if (!reader.close())
            {
                Debug.WriteLine("ファイルのクローズに失敗しました(" + fname + ")");
            }
        }

        /// <summary>
        /// 現在のメッセージを返します。
        /// </summary>
        /// <returns></returns>
        public Message getCurrentMessage()
        {
            return this.messages[this.num];
        }

        /// <summary>
        /// 現在のメッセージ詳細を返します。
        /// </summary>
        /// <returns></returns>
        public MessageDetail getCurrentMessageDetail()
        {
            return this.messages[this.num].message_detail[this.messages[this.num].num];
        }

        /// <summary>
        /// メッセージのメイン処理です。
        /// </summary>
        public void message_main()
        {
            KeyManager keyManager = KeyManager.GetInstance();
            if (this.is_displayed && this.pad_old == 0 && keyManager.CheckStatePad(GV.configpad.shot) > 0)
            {
                Message message = this.messages[this.num];
                message.num++;
                if (message.num == message.max_num)
                {
                    this.num++;
                    this.is_displayed = false;
                }
            }

            this.pad_old = keyManager.CheckStatePad(GV.configpad.shot);

            if (this.num < this.max_num) // メッセージを出し切っていなくて
            {
                Message message = this.messages[this.num];
                if (message.appear_count == GV.stage_count) // カウントが一緒で
                {
                    if (message.num < message.max_num) // 読みかけのメッセージがあれば
                    {
                        if (GV.stage_count != GV.boss_infos[GV.boss.num].appear_count) // ボスでなければ
                        {
                            if (!this.is_displayed)
                            {
                                // 敵と弾を消す
                                GV.enemys.initialize();
                                GV.shots.initialize();
                                this.cnt = 0;
                            }
                            this.is_displayed = true;
                        }
                        else // ボスであれば
                        {
                            if (GV.boss.cnt >= 60)
                            {
                                if (!this.is_displayed)
                                    this.cnt = 0;

                                this.is_displayed = true;
                            }
                        }
                    }
                }
            }

            this.cnt++;
        }
    }
}
